/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


/**
   File Name:          15.3.4-1.js
   ECMA Section:       15.3.4  Properties of the Function Prototype Object

   Description:        The Function prototype object is itself a Function
   object ( its [[Class]] is "Function") that, when
   invoked, accepts any arguments and returns undefined.

   The value of the internal [[Prototype]] property
   object is the Object prototype object.

   It is a function with an "empty body"; if it is
   invoked, it merely returns undefined.

   The Function prototype object does not have a valueOf
   property of its own; however it inherits the valueOf
   property from the Object prototype Object.

   Author:             christine@netscape.com
   Date:               28 october 1997

*/

var SECTION = "15.3.4-1";
var VERSION = "ECMA_1";
startTest();
var TITLE   = "Properties of the Function Prototype Object";

writeHeaderToLog( SECTION + " "+ TITLE);

new TestCase( SECTION,
	      "var myfunc = Function.prototype; myfunc.toString = Object.prototype.toString; myfunc.toString()",
	      "[object Function]",
	      eval("var myfunc = Function.prototype; myfunc.toString = Object.prototype.toString; myfunc.toString()"));


//  new TestCase( SECTION,  "Function.prototype.__proto__",     Object.prototype,           Function.prototype.__proto__ );

new TestCase( SECTION, 
	      "Function.prototype.valueOf",      
	      Object.prototype.valueOf,  
	      Function.prototype.valueOf );

new TestCase( SECTION, 
	      "Function.prototype()",            
	      (void 0),                  
	      Function.prototype() );

new TestCase( SECTION, 
	      "Function.prototype(1,true,false,'string', new Date(),null)", 
	      (void 0),
	      Function.prototype(1,true,false,'string', new Date(),null) );

test();
