/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Mozilla Communicator client code, released
 * March 31, 1998.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1998-2001
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *   Joe Hewitt <hewitt@netscape.com>
 *   Manuel Reimer <Manuel.Reimer@gmx.de>
 *   Stefan Hermes <stefanh@inbox.com>
 *   Jens Hatlak <jh@junetz.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

/* ===== addressbook.css ================================================
  == Styles for the main Address Book window.
  ======================================================================= */

@import url("chrome://messenger/skin/");

@namespace url("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul");
@namespace html url("http://www.w3.org/1999/xhtml");

#addressbookWindow {
  -moz-appearance: none;
  background-color: #EEEEEE;
}

/* ::::: primary toolbar buttons ::::: */

#button-newcard {
  list-style-image: url("chrome://messenger/skin/addressbook/icons/addressbookicons.png");
  -moz-image-region: rect(60px 29px 89px 0);
}

#button-newcard:hover:active  {
  -moz-image-region: rect(60px 59px 89px 30px);
} 

#button-newcard[disabled]{
  -moz-image-region: rect(60px 89px 89px 60px);
}

#button-newlist {
  list-style-image: url("chrome://messenger/skin/addressbook/icons/addressbookicons.png");
  -moz-image-region: rect(90px 29px 119px 0);
}

#button-newlist:hover:active {
  -moz-image-region: rect(90px 59px 119px 30px);
} 

#button-newlist[disabled] {
  -moz-image-region: rect(90px 89px 119px 60px);
}

#button-editcard {
  list-style-image: url("chrome://messenger/skin/addressbook/icons/addressbookicons.png");
  -moz-image-region: rect(30px 29px 59px 0);
}

#button-editcard:hover:active {
  -moz-image-region: rect(30px 59px 59px 30px);
} 

#button-editcard[disabled] {
  -moz-image-region: rect(30px 89px 59px 60px);
}

#button-newmessage {
  list-style-image: url("chrome://messenger/skin/icons/messengericons.png");
  -moz-image-region: rect(150px 29px 179px 0);
}

#button-newmessage:hover:active {
  -moz-image-region: rect(150px 59px 179px 30px);
} 

#button-newmessage[disabled]{
  -moz-image-region: rect(150px 89px 179px 60px);
} 

#button-newim {
  list-style-image: url("chrome://messenger/skin/addressbook/icons/addressbookicons.png");
  -moz-image-region: rect(120px 29px 149px 0);
}

#button-newim:hover:active {
  -moz-image-region: rect(120px 59px 149px 30px);
} 

#button-newim:hover[disabled] {
  -moz-image-region: rect(120px 89px 149px 60px);
}

#button-abdelete {
  list-style-image: url("chrome://messenger/skin/addressbook/icons/addressbookicons.png");
  -moz-image-region: rect(0 29px 29px 0);
}

#button-abdelete:hover:active  {
  -moz-image-region: rect(0 59px 29px 30px);
} 

#button-abdelete[disabled] {
  -moz-image-region: rect(0 89px 29px 60px);
}

#results-splitter {
  background-color: -moz-dialog;
  border-top: 1px solid #A5A5A5;
  border-bottom: 1px solid #A5A5A5;
}

#dirTree-splitter {
  border: solid #B3B3B3;
  border-width: 0 1px 0 0;
  background-image: none;
  min-width: 1px;
}

/* Make sure users find the splitter when it's collapsed... */

#dirTree-splitter[state="collapsed"] {
  min-width: 5px;
  background-color: #D6DDE5;
}

#dirTreeBox {
  background-color: #D6DDE5;
}

#dirTreeBox:-moz-window-inactive {
  background-color: #E8E8E8;
}

#dirTree {
  background-color: transparent;
}

#dirTree > treechildren::-moz-tree-row {
  background-color: transparent;
}

#dirTree > treechildren::-moz-tree-row(selected) {
  background-color: -moz-mac-secondaryhighlight;
}

#dirTree > treechildren::-moz-tree-row(selected, focus) {
  background-color: Highlight;
}

#DirCol {
  -moz-appearance: none;
  border: none;
  background-color: transparent;
}

#DirCol > .treecol-text {
  padding: 6px 0 6px 8px;
  font-weight: bold;
  color: #738193;
}

/* CardView styles - used in the Card View Pane */
 
#CardViewBox {
  -moz-user-focus: ignore;
  overflow: auto;
  min-width: 150px;
  background-color: #FFFFFF;
}

#CardViewInnerBox {
  margin-top: 2px;
  margin-bottom: 2px;
  padding: 0 8px;
}

#CardTitle {
  font-size: 150%;
  font-weight: bold;
  border-bottom: 2px solid black;
  min-width: 120px;
  margin: 0px;
}
 
.cardViewGroup {
  margin-top: 8px;
  padding-bottom: 5px;
  min-width: 50px;
}

.CardViewHeading {
  padding: 1px;
  -moz-padding-start: 5px;
  margin: 0px 0px 1px;
  background-color: -moz-dialog;
  color: black;
  font-weight: bold;
  min-width: 30px;
}

#cvBuddyIcon {
  -moz-padding-start: 20px;
  padding-top: 2px;
}

.CardViewText,
.CardViewLink {
  -moz-padding-start: 20px;
  -moz-padding-end: 2px;
  min-width: 30px;
  margin: 0px;
}

.CardViewLink {
  color: blue;
  text-decoration: underline;
  cursor: pointer;
}


#cvHomeMapIt, #cvWorkMapIt {
  margin-bottom: 0px;
}

html|a {
  -moz-padding-start: 0px;
  -moz-padding-end: 2px;
  border: none !important;
}

html|p {
  border: none !important;
}

/* ::::: lightweight themes ::::: */

#CardViewOuterBox:-moz-lwtheme {
  background-color: -moz-dialog;
  color: -moz-dialogtext;
  text-shadow: none
}
