/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is
 * Netscape Communications Corporation.
 * Portions created by the Initial Developer are Copyright (C) 1999
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */
package netscape.ldap.util;

/** This abstract class is the parent of those classes which implement
    MIME encoding and decoding: base64.
    @see netscape.ldap.util.MimeBase64Encoder
    @see netscape.ldap.util.MimeBase64Decoder
 */

public abstract class MimeEncoder implements java.io.Serializable {

    static final long serialVersionUID = 5179250095383961512L;

    /** Given a sequence of input bytes, produces a sequence of output bytes.
        Note that some (small) amount of buffering may be necessary, if the
        input byte stream didn't fall on an appropriate boundary.  If there
        are bytes in `out' already, the new bytes are appended, so the
        caller should do `out.setLength(0)' first if that's desired.
     */
    abstract public void translate(ByteBuf in, ByteBuf out);

    /** Tell the decoder that no more input data will be forthcoming.
        This may result in output, as a result of flushing the internal
        buffer.  This object must not be used again after calling eof().
        If there are bytes in `out' already, the new bytes are appended,
        so the caller should do `out.setLength(0)' first if that's desired.
     */
    abstract public void eof(ByteBuf out);
}
